/////////////////////////////////////////////////////////////////////////////
// ShapeCtl.cpp : Implementation of CShapeCtl
//
// A Simple ActiveX Control
//
// All rights reserved.
//
// Written by Naveen Kohli (naveen@a3ds.com)
// Version 1.0
//
// Distribute freely, except: don't remove my name from the source or
// documentation (don't take credit for my work), mark your changes (don't
// get me blamed for your possible bugs), don't alter or remove this
// notice.
// No warrantee of any kind, express or implied, is included with this
// software; use at your own risk, responsibility for damages (if any) to
// anyone resulting from the use of this software rests entirely with the
// user.
//
// Send bug reports, bug fixes, enhancements, requests, flames, etc. to
//    naveen@a3ds.com
/////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "ActiveXCtl.h"
#include "ShapeCtl.h"

/////////////////////////////////////////////////////////////////////////////
// CShapeCtl

static POINT CenterPt;
static long RectHt = 0;
static long RectWd = 0;

// IObjectSafety interface implementation

//
//	Function:
//		GetInterfaceSafetyOptions (REFIID riid, DWORD *pdwSupportedOptions,
//			DWORD *pdwEnabledOptions)
//
//	Parameters:
//		riid				Specifies the interface ID for a given object.
//		pdwSupportedOptions Specifies the options supported for the interface identified by riid.
//		pdwEnabledOptions	Specifies the options currently enabled for the interface identified
//							by riid. 
//
//	Return Values:
//		Returns S_OK if successful, or E_NOINTERFACE if riid specifies an interface that is unknown
//		to the object
//
//	Comments:
//		This method returns a set of bits in pdwSupportedOptions for each capability that the
//		control knows about, and a set of bits in pdwEnabledOptions for each capability for which
//		the control is currently safe. 
//
//	NOTE: There is a mistake in Microsoft's documentation on IObjectSafetyImpl interface. The
//		  member variable which gives the safety option for interface is m_dwCurrentSafety. But in
//		  the Knowledgebase articales and the online documentation it has been written as
//		  m_dwSafety.


STDMETHODIMP
CShapeCtl::GetInterfaceSafetyOptions (REFIID riid, DWORD *pdwSupportedOptions,
		DWORD *pdwEnabledOptions)
{
	HRESULT hRes = E_FAIL;
	LPUNKNOWN pUnknown;

	// Check if the interface even support this interface.

	hRes = _InternalQueryInterface (riid, (void **) &pUnknown);
	if (FAILED (hRes)) {
		ATLTRACE (_T ("Failed to get interface pointer!\n"));
		return (hRes);
	}

	// Make sure that the passed argument pointers are valid.
	if (pdwSupportedOptions == NULL || pdwEnabledOptions == NULL) {
		ATLTRACE (_T ("Null pointers in the arguments!\n"));
		return (hRes);
	}

	// Release the inteface pointer.

	pUnknown->Release ();
	pUnknown = NULL;

	// Check what category the iid belongs to.

	if (riid == IID_IDispatch) {
		// IDispatch is interface used for scripting. If the control supports IDispatch or Dual
		// interfaces, then mark the control as safe for scripting. Make sure that you don't play
		// with this option because user of your control relies on you.
		
		*pdwSupportedOptions = INTERFACESAFE_FOR_UNTRUSTED_CALLER;
		*pdwEnabledOptions = m_dwCurrentSafety & INTERFACESAFE_FOR_UNTRUSTED_CALLER;

		return (S_OK);
	}
	else if (riid == IID_IPersistStreamInit || riid == IID_IPersistStorage ||
		riid == IID_IPersistPropertyBag) {
		// The persistence interfaces are used for intialization. If other persistence interfaces
		// are supported also, add in this section. Indicate to user that control is safe for
		// initializing.

		*pdwSupportedOptions = INTERFACESAFE_FOR_UNTRUSTED_DATA;
		*pdwEnabledOptions = m_dwCurrentSafety & INTERFACESAFE_FOR_UNTRUSTED_DATA;

		return (S_OK);
	}
	else {
		// If iid belongs to none of the supported persistence or dispatch interface then
		// mark the control unsafe for initializing or scripting.

		*pdwSupportedOptions = 0;
		*pdwEnabledOptions = 0;

		return (E_FAIL);
	}

	return (hRes);
}

//
//	Function:
//		SetInterfaceSafetyOptions (REFIID riid, DWORD dwSupportedOptions,
//			DWORD dwEnabledOptions)
//
//	Parameters:
//		riid				Specifies the interface ID for a given object.
//		dwSupportedOptions	Specifies the options supported for the interface identified by riid.
//		dwEnabledOptions	Specifies the options currently enabled for the interface identified
//							by riid. 
//
//	Return Values:
//		S_OK			Success,  
//		E_NOINTERFACE	riid specifies an interface that is unknown to the object.  
//		E_FAIL			dwOptionSetMask specifies an option that is not supported by the object.  
//
//	Comments:
//		This method takes an interface (either IDispatch for scripting or IPersist* for
//		initialization), a bitmask of options to change (dwOptionSetMask), and the new settings for
//		those options (dwEnabledOptions). The currently supported capabilities are, as described
//		above, INTERFACESAFE_FOR_UNTRUSTED_CALLER and INTERFACESAFE_FOR_UNTRUSTED_DATA
//

STDMETHODIMP
CShapeCtl::SetInterfaceSafetyOptions (REFIID riid, DWORD dwOptionSetMask,
		DWORD dwEnabledOptions)
{
	HRESULT hRes = E_FAIL;
	LPUNKNOWN pUnknown;

	if (!dwOptionSetMask || !dwEnabledOptions) {
		ATLTRACE (_T ("Null arguments!\n"));
		return (hRes);
	}

	// Get the interface pointer and check if its supported or not.

	hRes = _InternalQueryInterface (riid, (void**) &pUnknown);
	if (FAILED (hRes)) {
		ATLTRACE (_T ("Interface not supported!\n"));
		return (hRes);
	}

	// Release the interface pointer.

	pUnknown->Release ();
	pUnknown = NULL;

	// Store the current safety level.

	m_dwCurrentSafety |= dwEnabledOptions & dwOptionSetMask;

	// check what kind of iterface to the passed iid belongs to.

	if ((riid == IID_IDispatch) && (m_dwCurrentSafety & INTERFACESAFE_FOR_UNTRUSTED_CALLER)) {
		// Client site is making this control safe for scripting. Control is safe for scripting
		// by default, therefore we return success flag.
	}
	else if (((riid == IID_IPersistStreamInit) || (riid == IID_IPersistStorage) ||
		(riid == IID_IPersistPropertyBag)) && (m_dwCurrentSafety &
		INTERFACESAFE_FOR_UNTRUSTED_DATA)) {
		// The client site is making this control safe for intitialization from persistent
		// object data. Since our control is safe for these persistence interfaces, we return
		// success flag.

		return (S_OK);
	}
	else {
		// The control doesn't allow any safe initialization or scripting for any other type of
		// interface.

		return (E_FAIL);
	}

	return (hRes);
}

// IOleControl interface implementation

//
//	Function:
//		OnAmbientPropertyChange (DISPID dispid)
//
//	Parameters:
//		dispID		Dispatch identifier of the ambient property that changed. If the dispid
//					parameter is DISPID_UNKNOWN, it indicates that multiple properties changed.
//					In this case, the control should check all the ambient properties of interest
//					to obtain their current values. 	 
//
//	Return Values:
//		S_OK		Returned in all cases
//
//	Comments:
//		Informs a control that one or more of the container's ambient properties (available through
//		the control site's IDispatch) has changed.
//
// NOTE : This interface retruns S_OK in all cases, even if some error has occured.
//

STDMETHODIMP
CShapeCtl::OnAmbientPropertyChange (DISPID dispid)
{
	// It is upto control to decide which ambient property it wants to respond to.

	// NOTE: In this control only change to background color is being acknowledges. If you want
	//		 to respond to any other anbient property, add the appropriate code at the appropriate
	//		 place in switch statement.

	switch (dispid) {
		case DISPID_AMBIENT_BACKCOLOR:
			// Get the ambient back color and change the view of control.
			if (SUCCEEDED (GetAmbientBackColor (m_clrBackColor))) {
				FireViewChange ();
			}
			break;
		case DISPID_AMBIENT_DISPLAYNAME:
		case DISPID_AMBIENT_FONT:
		case DISPID_AMBIENT_FORECOLOR:
		case DISPID_AMBIENT_LOCALEID:
		case DISPID_AMBIENT_MESSAGEREFLECT:
		case DISPID_AMBIENT_SCALEUNITS:
		case DISPID_AMBIENT_TEXTALIGN:
		case DISPID_AMBIENT_USERMODE:
		case DISPID_AMBIENT_UIDEAD:
		case DISPID_AMBIENT_SHOWGRABHANDLES:
		case DISPID_AMBIENT_SHOWHATCHING:
		case DISPID_AMBIENT_DISPLAYASDEFAULT:
		case DISPID_AMBIENT_SUPPORTSMNEMONICS:
		case DISPID_AMBIENT_AUTOCLIP:
		case DISPID_AMBIENT_APPEARANCE:
		case DISPID_AMBIENT_CODEPAGE:
		case DISPID_AMBIENT_PALETTE:
		case DISPID_AMBIENT_CHARSET:
		case DISPID_AMBIENT_TRANSFERPRIORITY:
		case DISPID_AMBIENT_RIGHTTOLEFT:
		case DISPID_AMBIENT_TOPTOBOTTOM:
		case DISPID_UNKNOWN:
			break;
	}

	return (S_OK);
}

// ISupportsErrorInfo

//
//	Function:
//		InterfaceSupportsErrorInfo (REFIID riid)
//
//	Parameters:
//		riid 		Pointer to an interface identifier (IID). 
//
//	Return Values:
//		S_OK		Interface supports IErrorInfo. 
//		S_FALSE		Interface does not support IErrorInfo. 

//
//	Comments:
//		Indicates whether or not an interface supports the IErrorInfo interface.
//

STDMETHODIMP
CShapeCtl::InterfaceSupportsErrorInfo (REFIID riid)
{
	static const IID* arr[] = 
	{
		&IID_IShapeCtl,
	};
	for (int i=0; i<sizeof(arr)/sizeof(arr[0]); i++)
	{
		if (InlineIsEqualGUID(*arr[i], riid))
			return S_OK;
	}
	return S_FALSE;
}

// IPersistStream

//
//	Function:
//		Load (IStream *pStream)
//
//	Parameters:
//		pStream 		IStream pointer to the stream from which the object should be loaded. 
//
//	Return Values:
//		S_OK			The object was successfully loaded. 
//		E_OUTOFMEMORY	The object was not loaded due to a lack of memory. 
//		E_FAIL			The object was not loaded due to some reason other than a lack of memory. 
//
//	Comments:
//		Initializes an object from the stream where it was previously saved. On exit, the seek
//		pointer should be left in the same position as it was on exit from Save, immediately past
//		the end of the data.
//
//	NOTE: If the object has already been initialized with InitNew, then this method must return
//		  E_UNEXPECTED
//

STDMETHODIMP
CShapeCtl::Load (IStream *pStream)
{
	DWORD nActual;
	HRESULT hRes = E_UNEXPECTED;

	// Check if the passed interface pointer is valid or not.

	if (NULL == pStream) {
		return (E_POINTER);
	}

	// Load each parameter value from the stream. Make sure that the order of loading is same as
	// the order inwhich parameters were saved to stream.

	pStream->Read (&m_nRadius, sizeof (short), &nActual);
	pStream->Read (&m_clrFillColor, sizeof (OLE_COLOR), &nActual);
	pStream->Read (&m_clrBackColor,sizeof (OLE_COLOR), &nActual);
	pStream->Read (&m_nBackStyle, sizeof (LONG), &nActual);
	pStream->Read (&m_clrBorderColor, sizeof (OLE_COLOR), &nActual);
	pStream->Read (&m_nBorderStyle, sizeof (LONG), &nActual);
	pStream->Read (&m_bBorderVisible, sizeof (BOOL), &nActual);
	pStream->Read (&m_nBorderWidth, sizeof (LONG), &nActual);
	pStream->Read (&m_nDrawMode, sizeof (LONG), &nActual);
	pStream->Read (&m_nDrawStyle, sizeof (LONG), &nActual);
	pStream->Read (&m_nDrawWidth, sizeof (LONG), &nActual);
	pStream->Read (&m_bEnabled, sizeof (BOOL), &nActual);
	pStream->Read (&m_nFillStyle, sizeof (LONG), &nActual);
	pStream->Read (&m_clrForeColor, sizeof (OLE_COLOR), &nActual);
	pStream->Read (&m_bValid, sizeof (BOOL), &nActual);
	pStream->Read (&m_nFillPattern, sizeof (m_nFillPattern), &nActual);

	// NOTE: Make sure that you set the m_str member variable to NULL. This has to be dome to avoid
	//		 assertion inside the ATL code.
	//
	//		 Code from ATLBASE.H file...................
	//		 **********************************************************
	//		 HRESULT ReadFromStream(IStream* pStream)
	//		 {
	//			ATLASSERT(pStream != NULL);
	//			ATLASSERT(m_str == NULL); // should be empty
	//			..
	//		 *********************************************************	
	//

	m_bstrText.m_str = NULL;
	m_bstrText.ReadFromStream (pStream);

	// Since IPersistStream interface is implemented by IFont, make use of it to persist the
	// font property.

	IPersistStream *pFontStream;

	hRes = m_pFont->QueryInterface (IID_IPersistStream, (void **) &pFontStream);
	if (FAILED (hRes)) {
		Error (_T ("Failed to create stream interface for font!"));
		return (E_FAIL);
	}

	hRes = pFontStream->Load (pStream);
	if (FAILED (hRes)) {
		Error (_T ("Failed to create stream interface for font!"));
		return (E_FAIL);
	}

	// Release the interface poniter.

	pFontStream->Release ();

	return (S_OK);
}

//
//	Function:
//		Save(IStream *pStream, BOOL fClearDirty)
//
//	Parameters:
//		pStream 		IStream pointer to the stream into which the object should be saved.  
//		fClearDirty		Indicates whether to clear the dirty flag after the save is complete. If
//						TRUE, the flag should be cleared. If FALSE, the flag should be left
//						unchanged.
//
//	Return Values:
//		S_OK			The object was successfully saved to the stream.
//		STG_E_CANTSAVE 	The object could not save itself to the stream. This error could indicate,
//						for example, that the object contains another object that is not
//						serializable to a stream or that an ISequentialStream::Write call returned
//						STG_E_CANTSAVE. 
//		STG_E_MEDIUMFULL The object could not be saved because there is no space left on the
//						 storage device. 
//		E_FAIL			Error
//
//	Comments:
//		Saves an object into the specified stream and indicates whether the object should reset
//		its dirty flag.
//

STDMETHODIMP
CShapeCtl::Save(IStream *pStream, BOOL fClearDirty)
{
	DWORD nActual, nToWrite;
	HRESULT hRes = E_FAIL;

	// Make sure that IStream pointer is not null.

	if (pStream == NULL) {
		ATLTRACE (_T ("Null stream pointer!\n"));
		return (E_POINTER);
	}

	// Write the control parameters to the stream.
	// NOTE: Following implementation doesn't check for the difference between size to be saved
	//		 and the size actually saved to report error. Its being done for only first one. The
	//		 error for the other properties can be checked in same way.

	nToWrite = sizeof (m_nRadius);
	hRes = pStream->Write (&m_nRadius, nToWrite, &nActual);
	if (SUCCEEDED (hRes) && nToWrite != nActual) {
		return (STG_E_CANTSAVE);
	}

	pStream->Write (&m_clrFillColor, sizeof (OLE_COLOR), &nActual);
	pStream->Write (&m_clrBackColor,sizeof (OLE_COLOR), &nActual);
	pStream->Write (&m_nBackStyle, sizeof (LONG), &nActual);
	pStream->Write (&m_clrBorderColor, sizeof (OLE_COLOR), &nActual);
	pStream->Write (&m_nBorderStyle, sizeof (LONG), &nActual);
	pStream->Write (&m_bBorderVisible, sizeof (BOOL), &nActual);
	pStream->Write (&m_nBorderWidth, sizeof (LONG), &nActual);
	pStream->Write (&m_nDrawMode, sizeof (LONG), &nActual);
	pStream->Write (&m_nDrawStyle, sizeof (LONG), &nActual);
	pStream->Write (&m_nDrawWidth, sizeof (LONG), &nActual);
	pStream->Write (&m_bEnabled, sizeof (BOOL), &nActual);
	pStream->Write (&m_nFillStyle, sizeof (LONG), &nActual);
	pStream->Write (&m_clrForeColor, sizeof (OLE_COLOR), &nActual);
	pStream->Write (&m_bValid, sizeof (BOOL), &nActual);
	pStream->Write (&m_nFillPattern, sizeof (m_nFillPattern), &nActual);
	m_bstrText.WriteToStream (pStream);

	// Since IPersistStream interface is implemented by IFont, make use of it to persist the
	// font property.

	IPersistStream *pFontStream;

	hRes = m_pFont->QueryInterface (IID_IPersistStream, (void **) &pFontStream);
	if (FAILED (hRes)) {
		Error (_T ("Failed to create stream interface for font!"));
		return (E_FAIL);
	}

	hRes = pFontStream->Save (pStream, true);
	if (FAILED (hRes)) {
		Error (_T ("Failed to create stream interface for font!"));
		return (E_FAIL);
	}

	SetDirty (!fClearDirty);
	pFontStream->Release ();

	return (S_OK);
}

//
//	Function:
//		GetSizeMax (ULARGE_INTEGER *pcbSize)
//
//	Parameters:
//		pcbSize   		Pointer to size of stream needed to save object .
//
//	Return Values:
//		S_OK			The size was successfully returned.  
//		E_FAIL			The size was not successfully returned. 
//
//	Comments:
//		This method returns the size needed to save an object. You can call this method to
//		determine the size and set the necessary buffers before calling the
//		IPersistStream::Save method
//
//	NOTE: The followinf MACRO from OBJBASE.H is used to set the size in ULARGE_INTEGER
//			#define ULISet32(li, v) ((li).HighPart = 0, (li).LowPart = (v))
//

STDMETHODIMP
CShapeCtl::GetSizeMax (ULARGE_INTEGER *pcbSize)
{
	HRESULT hRes = E_POINTER;
	ULONG ulsize;

	// Make sure that pointer is not null.

	if (pcbSize == NULL) {
		return (hRes);
	}

	// Add the size of each property.

	ulsize = sizeof (m_nRadius);
	ulsize += sizeof (m_clrFillColor);
	ulsize += sizeof (m_nBackStyle);
	ulsize += sizeof (m_clrBorderColor);
	ulsize += sizeof (m_nBorderStyle);
	ulsize += sizeof (m_bBorderVisible);
	ulsize += sizeof (m_nBorderWidth);
	ulsize += sizeof (m_nDrawMode);
	ulsize += sizeof (m_nDrawStyle);
	ulsize += sizeof (m_nDrawWidth);
	ulsize += sizeof (m_bEnabled);
	ulsize += sizeof (m_nFillStyle);
	ulsize += sizeof (m_clrForeColor);
	ulsize += sizeof (m_bValid);
	ulsize += sizeof (m_nFillPattern);
	ulsize += sizeof (OLECHAR) * m_bstrText.Length ();

	// Since IPersistStream interface is implemented by IFont, make use of it to get the size
	// to persist this property.

	IPersistStream *pFontStream;

	hRes = m_pFont->QueryInterface (IID_IPersistStream, (void **) &pFontStream);
	if (FAILED (hRes)) {
		Error (_T ("Failed to create stream interface for font!"));
		return (E_FAIL);
	}

	hRes = pFontStream->GetSizeMax (pcbSize);
	if (FAILED (hRes)) {
		Error (_T ("Failed to create stream interface for font!"));
		return (E_FAIL);
	}

	ulsize += pcbSize->LowPart;

	// Set the size of control.

	ULISet32 (*pcbSize, ulsize);

	return (S_OK);
}

// IPersistStorage

//
//	Function:
//		Load (IStorage* pStorage)
//
//	Parameters:
//		pStorage 		Pointer to existing storage for the object
//
//	Return Values:
//		S_OK			The object was successfully loaded.
//		CO_E_ALREADYINITIALIZED  The object has already been initialized by a previous call to the
//								 IPersistStorage::Load method or the IPersistStorage::InitNew
//								 method. 
//		E_OUTOFMEMORY	The object was not loaded due to a lack of memory. 
//		E_FAIL			The object was not loaded due to some reason other than a lack of memory. 
//
//	Comments:
//		This method initializes an object from an existing storage. The object is placed in the
//		loaded state if this method is called by the container application. If called by the
//		default handler, this method places the object in the running state.
//
//	NOTE:
//		The implementation should perform the following steps to load an object: 
//		1) Open the object's streams in the storage object, and read the necessary data into the
//		object's internal data structures.
//		2) Clear the object's dirty flag. 
//		3) Call the IPersistStorage::AddRef method and cache the passed in storage pointer. 
//		4) Keep open and cache the pointers to any streams or storages that the object will need to
//		   save itself to this storage. 
//		5) Perform any other default initialization required for the object. 
//		
//		Steps 3 and 4 are particularly important for ensuring that the object can save itself in
//		low memory situations. Holding onto pointers to the storage and stream interfaces
//		guarantees that a save operation to this storage will not fail due to insufficient memory.

STDMETHODIMP
CShapeCtl::Load (IStorage* pStorage)
{
	DWORD nActual;
	HRESULT hRes = E_UNEXPECTED;
	CComQIPtr <IStream, &IID_IStream> pPropStream;
	CComQIPtr <IStream, &IID_IStream> pDataStream;
	CComQIPtr <IStream, &IID_IStream> pSummaryStream;
	CComQIPtr <IStorage, &IID_IStorage> pSubStorage;

	if (NULL == pStorage) {
		return (E_FAIL);
	}

	// Open stream.
	
	hRes = pStorage->OpenStream (L"Shape Control Data", NULL, STGM_READWRITE |
		STGM_SHARE_EXCLUSIVE, NULL, &pPropStream);
	if (FAILED (hRes)) {
		return (E_FAIL);
	}
	
	// open the  sub storage.

	hRes = pStorage->OpenStorage (L"Summary Info",NULL, STGM_READWRITE | STGM_TRANSACTED |
		STGM_SHARE_EXCLUSIVE, NULL, 0, &pSubStorage);
	if (FAILED (hRes)) {
		return (E_FAIL);
	}

	// Create a stream to contain summary information about the control.

	hRes = pStorage->OpenStream (L"Summary Stream", NULL, STGM_READWRITE | STGM_SHARE_EXCLUSIVE,
		NULL, &pSummaryStream);
	if (FAILED (hRes)) {
		ATLTRACE (_T ("Failed to create summary stream in sub storage."));
		return (E_FAIL);
	}

	// Write some onformation inthis stream.

	pSummaryStream->Read (&m_bstrAuthor, sizeof (m_bstrAuthor), &nActual);
	pSummaryStream->Read (&m_bstrEmail, sizeof (m_bstrEmail), &nActual);
	pSummaryStream->Read (&m_bstrDesc, sizeof (m_bstrDesc), &nActual);

	// Create a new stream in substorage to write control parameters..

	hRes = pSubStorage->OpenStream (L"Control Parameters", NULL, STGM_READWRITE |
		STGM_SHARE_EXCLUSIVE, NULL, &pDataStream);
	if (FAILED (hRes)) {
		return (E_FAIL);
	}

	// Delegate the stream save to the Save function call for IStream..

	if (FAILED (Load (pDataStream))) {
		ATLTRACE (_T ("Failed to save the stream!\n"));
		return (E_FAIL);
	}

	// Clear the dirty flag.

	SetDirty (FALSE);

	// Keep open storage pointer around for use in later incremental or low-memory calls of save.

	m_pIStorage = pStorage;
	m_pIStorage->AddRef ();

	return (S_OK);
}

//
//	Function:
//		Save(IStorage* pStorage, BOOL fSameAsLoad)
//
//	Parameters:
//		pStorage 		IStream pointer to storage object.  
//		fSameAsLoad		Indicates whether the specified storage object is the current one
//
//	Return Values:
//		S_OK			The object was successfully saved to the stream.
//		STG_E_MEDIUMFULL The object could not be saved because there is no space left on the
//						 storage device. 
//		E_FAIL			The object could not be saved due to errors other than a lack of disk space
//
//	Comments:
//		This method saves an object, and any nested objects it contains, into the specified
//		storage. It also places the object into NoScribble mode.
//

STDMETHODIMP
CShapeCtl::Save(IStorage* pStorage, BOOL fSameAsLoad)
{
	DWORD nActual;
	HRESULT hRes = E_UNEXPECTED;
	CComQIPtr <IStream, &IID_IStream> pPropStream;
	CComQIPtr <IStream, &IID_IStream> pDataStream;
	CComQIPtr <IStream, &IID_IStream> pSummaryStream;
	CComQIPtr <IStorage, &IID_IStorage> pSubStorage;

	// Create stream.

	if (NULL == pStorage) {
		return (E_FAIL);
	}
	
	// Create a stream to save the information about control. Make sure that the names with which
	// the streams or substorages are created are the same which are used to load the object.

	hRes = pStorage->CreateStream (L"Shape Control Data", STGM_CREATE | STGM_WRITE |
		STGM_SHARE_EXCLUSIVE, 0, 0, &pPropStream);
	if (FAILED (hRes)) {
		return (E_FAIL);
	}
	
	// Create a sub storage.

	hRes = pStorage->CreateStorage (L"Summary Info", STGM_CREATE | STGM_WRITE |
		STGM_SHARE_EXCLUSIVE, 0, 0, &pSubStorage);
	if (FAILED (hRes)) {
		return (E_FAIL);
	}

	// Create a stream to contain summary information about the control.

	hRes = pStorage->CreateStream (L"Summary Stream", STGM_CREATE | STGM_WRITE |
		STGM_SHARE_EXCLUSIVE, 0, 0, &pSummaryStream);
	if (FAILED (hRes)) {
		ATLTRACE (_T ("Failed to create summary stream in sub storage."));
		return (E_FAIL);
	}

	// Write some onformation inthis stream.

	pSummaryStream->Write (&m_bstrAuthor, sizeof (m_bstrAuthor), &nActual);
	pSummaryStream->Write (&m_bstrEmail, sizeof (m_bstrEmail), &nActual);
	pSummaryStream->Write (&m_bstrDesc, sizeof (m_bstrDesc), &nActual);

	// Create a new stream in substorage to write control parameters..

	hRes = pSubStorage->CreateStream (L"Control Parameters", STGM_WRITE | STGM_WRITE |
	STGM_SHARE_EXCLUSIVE, 0, 0, &pDataStream);
	if (FAILED (hRes)) {
		return (E_FAIL);
	}

	// Delegate the stream save to the Save function call for IStream..

	if (FAILED (Save (pDataStream, true))) {
		ATLTRACE (_T ("Failed to save the stream!\n"));
		return (E_FAIL);
	}

	return (S_OK);
}

// IPeristPropertBag implementation.
//
//	Function:
//		Load (IPropertyBag *pPropBag, IErrorLog* pErrorLog)
//
//	Parameters:
//		pPropBag 		Pointer to caller's property bag
//		pErrorLog		Pointer to error log
//
//	Return Values:
//		S_OK			The object successfully initialized itself.
//		E_OUTOFMEMORY	The object was not loaded due to a lack of memory.
//		E_POINTER		The address in pPropBag is not valid (such as NULL) and therefore the
//						object cannot initialize itself. 
//		E_FAIL			The object was unable to retrieve a critical property that is necessary
//						for the object's successful operation. The object was therefore unable to
//						initialize itself completely. 
//
//	Comments:
//		Called by the container to load the control's properties. This method instructs the object
//		to initialize itself using the properties available in the property bag, notifying the
//		provided error log object when errors occur. All property storage must take place within
//		this method call as the object cannot hold the IPropertyBag pointer.
//		E_NOTIMPL is not a valid return code as any object implementing this interface must support
//		the entire functionality of the interface.
//
//	NOTE: The ATL_PROPMAP_ENTRY structure used by ATL is a s follows.
//			struct ATL_PROPMAP_ENTRY
//			{
//				LPCOLESTR szDesc;
//				DISPID dispid;
//				const CLSID* pclsidPropPage;
//				const IID* piidDispatch;
//				DWORD dwOffsetData;
//				DWORD dwSizeData;
//				VARTYPE vt;
//			}
//

STDMETHODIMP
CShapeCtl::Load (IPropertyBag *pPropBag, IErrorLog* pErrorLog)
{
	HRESULT hRes = E_UNEXPECTED;

	if (pPropBag == NULL) {
		return (E_POINTER);
	}

	// ATL implementation uses property map to check which properties need to be loaded from or
	// saved to property bag. The data is written to propertybag as a VARIANT. The default
	// implementation supports only the integer type data. If the vt memeber of ATL_PROPMAP_ENTRY
	// is of any other type, ATL ignores that map entry. Therfore we need to implement persistence
	// of non-variant data by generic functions which convert the data to and fro from VARIANT
	// type, so that we can persist that data. To make this happen, set the non-variant data type
	// to onw whichis not supported bt ATL.
	
	//	NOTE: For more information see "Professional ATL COM Programming, Wrox Press Ltd" by
	//		  Dr. Richard Grimes.
	//

	ATL_PROPMAP_ENTRY *pMap = GetPropertyMap ();
	ATLASSERT (pMap != NULL);

	for (int i = 0; pMap[i].pclsidPropPage != NULL; ++i) {
		if ((pMap[i].dwSizeData == 0) || (pMap[i].szDesc == NULL)) {
			continue;
		}

		// check variant data type.

		switch (pMap[i].vt) {
			case VTEX_FIXED_SIZE_BSTR_BLOB:
				CComVariant var;
				hRes = pPropBag->Read (pMap[i].szDesc, &var, pErrorLog);
				if (FAILED (hRes)) {
					return (hRes);
				}

				LPBYTE pData = reinterpret_cast<BYTE*>(pMap[i].dwOffsetData +
					reinterpret_cast<DWORD>(this));
				int size = pMap[i].dwSizeData;
				LPCWSTR ptr = var.bstrVal;
				Extract (&ptr, pData, size);
				break;
		}
	}

	return (IPersistPropertyBagImpl<CShapeCtl>::Load (pPropBag, pErrorLog));
}

//
//	Function:
//		Save (IPropertyBag *pPropBag, BOOL fClearDirty, BOOL fSaveAllProperties)
//
//	Parameters:
//		pPropBag 			Pointer to caller's property bag
//		fClearDirty			Specifies whether to clear the dirty flag
//		fSaveAllProperties	Specifies whether to save all properties or just those that have
//							changed
//
//	Return Values:
//		S_OK			The object successfully initialized itself.
//		E_POINTER		The address in pPropBag is not valid (such as NULL) and therefore the
//						object cannot initialize itself. 
//		E_FAIL			There was a problem saving one of the properties. The object can choose to
//						fail only if a necessary property could not be saved, meaning that the
//						object can assume default property values if a given property is not seen
//						through IPersistPropertyBag::Load at some later time.  
//		STG_E_MEDIUMFULL The object was not saved because of a lack of space on the disk. 
//
//	Comments:
//		This method instructs the object to save its properties to the specified property bag,
//		optionally clearing the object's dirty flag. The caller can request that the object save
//		all properties or that the object save only those that are known to have changed.
//		E_NOTIMPL is not a valid return code as any object implementing this interface must support
//		the entire functionality of the interface.
//

STDMETHODIMP
CShapeCtl::Save (IPropertyBag *pPropBag, BOOL fClearDirty, BOOL fSaveAllProperties)
{
	HRESULT hRes = E_UNEXPECTED;

	if (pPropBag == NULL) {
		return (E_FAIL);
	}

	// See the discussion in Load function body.

	ATL_PROPMAP_ENTRY *pMap = GetPropertyMap ();
	ATLASSERT (pMap != NULL);

	for (int i = 0; pMap[i].pclsidPropPage != NULL; ++i) {
		if ((pMap[i].dwSizeData == 0) || (pMap[i].szDesc == NULL)) {
			continue;
		}

		// check variant data type.

		switch (pMap[i].vt) {
			case VTEX_FIXED_SIZE_BSTR_BLOB:
				LPBYTE pData = reinterpret_cast<BYTE*>(pMap[i].dwOffsetData +
					reinterpret_cast<DWORD>(this));
				CComBSTR bstr;
				Insert (&bstr, pData, pMap[i].dwSizeData);
				CComVariant var(bstr);
				hRes = pPropBag->Write (pMap[i].szDesc, &var);
				if (FAILED (hRes)) {
					return (hRes);
				}
				break;
		}
	}

	return (IPersistPropertyBagImpl<CShapeCtl>::Save (pPropBag, fClearDirty, fSaveAllProperties));
}

//
//	Function:
//		get_FillPattern(FILLPATTERN *pVal)
//
//	Parameters:
//		pVal	Pointer to fill pattern
//
//	Return Values:
//		S_OK	Fill pattern value returned successfully
//		E_FAIL	Failed to return fill pattern value
//
//	Comments:
//		Function returns the fill pattern of the control.
//

STDMETHODIMP
CShapeCtl::get_FillPattern(FILLPATTERN *pVal)
{
	*pVal = m_nFillPattern;

	return S_OK;
}

//
//	Function:
//		put_FillPattern(FILLPATTERN newVal)
//
//	Parameters:
//		newVal	Fill pattern type
//
//	Return Values:
//		S_OK	Fill pattern set successfully
//		E_FAIL	Failed to set fill pattern value
//
//	Comments:
//		Function sets the new value for the fill pattern of the control.
//

STDMETHODIMP
CShapeCtl::put_FillPattern(FILLPATTERN newVal)
{
	m_nFillPattern = newVal;

	FireViewChange ();

	return S_OK;
}

//
//	Function:
//		get_Radius(short *pVal)
//
//	Parameters:
//		pVal	Pointer to radius value
//
//	Return Values:
//		S_OK	Radius value returned successfully
//		E_FAIL	Failed to return radius value
//
//	Comments:
//		Function returns the radius value of the control.
//

STDMETHODIMP
CShapeCtl::get_Radius(short *pVal)
{
	*pVal = m_nRadius;
	SendOnDataChange ();

	return S_OK;
}

//
//	Function:
//		put_Radius(short newVal)
//
//	Parameters:
//		newVal	Radius of control
//
//	Return Values:
//		S_OK	Control radius set successfully
//		E_FAIL	Failed to set control radius
//
//	Comments:
//		Function sets the new value for the radius of the control.
//

STDMETHODIMP
CShapeCtl::put_Radius(short newVal)
{
	if (newVal <= 0) {
		return (Error (_T ("Radius value can't be zero!")));
	}
	else if (newVal > RectHt || newVal > RectWd) {
		return (Error (_T ("Radius can't exceed control's extents!")));
	}

	m_nRadius = newVal;

	// Change the control's view.

	FireViewChange ();

	return S_OK;
}

//
//	Function:
//		OnDraw(ATL_DRAWINFO& di)
//
//	Parameters;
//		di		A reference to the ATL_DRAWINFO structure that contains drawing information such
//				as the draw aspect, the control bounds, and whether the drawing is optimized or
//				not.
//
//	Return Values:
//		S_OK	Control was drawn successfully
//		E_FAIL	Failed to draw control
//
//	Comments:
//		Function draws the control with the specified parameters and property values.
//

HRESULT
CShapeCtl::OnDraw(ATL_DRAWINFO& di)
{
	USES_CONVERSION;
	HBRUSH hOldBrush, hBrush;
	HPEN hOldPen, hPen;
	HFONT hOldFont, hFont;
	OLE_COLOR colorFill, colorBorder, colorBack;

	HDC hdc = di.hdcDraw;

	RECT& rc = *(RECT*)di.prcBounds;
	
	// Save the center point.

	CenterPt.x = (long)((rc.left + rc.right) / 2.0);
	CenterPt.y = (long)((rc.top + rc.bottom) / 2.0);
	
	// Calculate height and width of control rectangle.

	RectWd = rc.right - rc.left;
	RectHt = rc.bottom - rc.top ;

	OleTranslateColor (m_clrBackColor, NULL, &colorBack);

	hBrush = (HBRUSH)GetStockObject (WHITE_BRUSH);
	hOldBrush = (HBRUSH)SelectObject (hdc, hBrush);

	hBrush = CreateSolidBrush (colorBack);

	SelectObject (hdc, hBrush);

	// Create the background rectangle.

	Rectangle (hdc, rc.left, rc.top, rc.right, rc.bottom);

	// Translate the fill color for control.

	OleTranslateColor (m_clrFillColor, NULL, &colorFill);

	// Save the old brush before we create new one to fill the shape.

	hBrush = (HBRUSH)GetStockObject (WHITE_BRUSH);
	hOldBrush = (HBRUSH)SelectObject (hdc, hBrush);

	// Create new brush with user specified color.

	switch (m_nFillStyle) {
		case 0:
		case 1:
			hBrush = CreateSolidBrush (colorFill);
			break;
		case 2:
			switch (m_nFillPattern) {
				case fillBDiagonal:
					hBrush = CreateHatchBrush (HS_BDIAGONAL, colorFill); 
					break;
				case fillCrossHatch:
					hBrush = CreateHatchBrush (HS_CROSS, colorFill);
					break;
				case fillDiagXHatch:
					hBrush = CreateHatchBrush (HS_DIAGCROSS, colorFill);
					break;
				case fillFDiagonal:
					hBrush = CreateHatchBrush (HS_FDIAGONAL, colorFill);
					break;
				case fillHorizontal:
					hBrush = CreateHatchBrush (HS_HORIZONTAL, colorFill);
					break;
				case fillVertical:
					hBrush = CreateHatchBrush (HS_VERTICAL, colorFill);
					break;
			}
			break;
	}
	
	SelectObject (hdc, hBrush);

	// Save the old pen before we create new one.

	hPen = (HPEN)GetStockObject (BLACK_PEN);
	hOldPen = (HPEN)SelectObject (hdc, hPen);

	OleTranslateColor (m_clrBorderColor, NULL, &colorBorder);

	// Create the new pen.

	switch (m_nDrawStyle) {
		case PS_SOLID:
		case PS_NULL:
		case PS_INSIDEFRAME:
			hPen = CreatePen (m_nDrawStyle, m_nDrawWidth, colorBorder);
			break;
		default:
			hPen = CreatePen (m_nDrawStyle, 0, colorBorder);
			break;
	}

	SelectObject (hdc, hPen);

	// Set the user specified font.

	CComQIPtr<IFont, &IID_IFont> pFont (m_pFont);

	if (pFont != NULL) {
		pFont->get_hFont (&hFont);
		pFont->AddRefHfont (hFont);
		hOldFont = (HFONT)SelectObject (hdc, hFont);
	}

	// Draw the circle.

	Ellipse (hdc, (CenterPt.x - m_nRadius), (CenterPt.y - m_nRadius), (CenterPt.x +m_nRadius),
		(CenterPt.y + m_nRadius));

	SetBkMode(hdc, TRANSPARENT);
	SetTextAlign(di.hdcDraw, TA_CENTER|TA_BASELINE);
	LPCTSTR pszText = OLE2T ((BSTR)m_bstrText);
	TextOut(di.hdcDraw, 
		(rc.left + rc.right) / 2, 
		(rc.top + rc.bottom) / 2, 
		pszText, 
		lstrlen(pszText));

	// Delete created GDI objects.

	if (pFont != NULL) {
		pFont->ReleaseHfont (hFont);
	}

	SelectObject (hdc, hOldFont);
	SelectObject (hdc, hOldBrush);
	DeleteObject (hBrush);
	SelectObject (hdc, hOldPen);
	DeleteObject (hPen);

	return S_OK;
}

// Message handler functions.

//
//	Function:
//		OnLButtonDblClk(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
//
//	Comments:
//		Function handles the left button double click message. It checks if the click is inside or
//		outside the control region. If the click is inside, ClickIn event is fired otherwise
//		ClickOut event is fired.
//

LRESULT
CShapeCtl::OnLButtonDblClk(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{
	HRGN rgn;
	WORD xPos = LOWORD (lParam);
	WORD yPos = HIWORD (lParam);

	// Create the elliptic region to determine if the click point was inside or outside the
	// control.

	rgn = CreateEllipticRgn((CenterPt.x - m_nRadius), (CenterPt.y - m_nRadius), (CenterPt.x + m_nRadius),
		(CenterPt.y + m_nRadius));

	// If the click was inside control, them fire ClickIn otherwise fire ClickOut.

	if (PtInRegion (rgn, xPos, yPos)) {
		Fire_ClickIn (xPos, yPos);
	}
	else {
		Fire_ClickOut (xPos, yPos);
	}

	// Deleted the created region object.

	DeleteObject (rgn);

	return 0;
}

//
//	Function:
//		OnLButtonDown(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
//
//	Comments:
//		Function handles the left button down message. It checks if the click is inside or outside
//		the control region. If the click is inside, ClickIn event is fired otherwise ClickOut
//		event is fired.
//

LRESULT
CShapeCtl::OnLButtonDown(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{
	HRGN rgn;
	WORD xPos = LOWORD (lParam);
	WORD yPos = HIWORD (lParam);

	// Create the elliptic region to determine if the click point was inside or outside the
	// control.

	rgn = CreateEllipticRgn((CenterPt.x - m_nRadius), (CenterPt.y - m_nRadius), (CenterPt.x + m_nRadius),
		(CenterPt.y + m_nRadius));

	// If the click was inside control, them fire ClickIn otherwise fire ClickOut.

	if (PtInRegion (rgn, xPos, yPos)) {
		Fire_ClickIn (xPos, yPos);
	}
	else {
		Fire_ClickOut (xPos, yPos);
	}

	// Deleted the created region object.

	DeleteObject (rgn);

	return 0;
}

//
//	Function:
//		OnMouseMove(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
//
//	Comments:
//		Function handles the mouse move message. For now it does nothing
//

LRESULT
CShapeCtl::OnMouseMove(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{
	// TODO: Add your code to handle mouse move message.
	return 0;
}

//
//	Function:
//		OnCreate(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
//
//	Comments:
//		Function handles the OnCreate message for the control. This function can be used to do
//		initializations for the control.
//

LRESULT
CShapeCtl::OnCreate(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{
	HRESULT hRes;
	HWND hWnd = NULL;
	HINSTANCE hInst;

	// If our control need the handle to control site's window, we can grab the pointer to 
	// IOleInPlaceSite interface and call GetWindow function to get the window pointer.

	CComQIPtr<IOleControlSite, &IID_IOleControlSite> pSite (m_spClientSite);
	CComQIPtr<IOleInPlaceSite, &IID_IOleInPlaceSite> pPlaceSite (m_spClientSite);

	hRes = pPlaceSite->GetWindow (&hWnd);
	if (FAILED (hRes)) {
		ATLTRACE (_T ("Get window call failed!\n"));
	}

	// Handle to control's instance can be obtained by calling GetModuleInstance call on the
	// module (control) itself.

	hInst = _Module.GetModuleInstance ();

	return (S_OK);
}

//
//	Function:
//		Insert (BSTR *pStream, void *pData, DWORD size)
//
//	Parameters:
//		pStream		Pointer to BSTR
//		pData		Pointer to buffer
//		size		Size of buffer in byte
//
//	Returns Values:
//
//	Comments:
//		Function converts each byte in the buffer to a two-character readable hexadecimal string,
//		and adds onto the end of the BSTR (CComBSTR class reallocates as neccessary)
//
//	NOTE: For more information see "Professional ATL COM Programming, Wrox Press Ltd" by
//		  Dr. Richard Grimes.
//

void
CShapeCtl::Insert (BSTR *pStream, void *pData, DWORD size)
{
	CComBSTR bstr;
	bstr.Attach (*pStream);

	LPBYTE ptr = static_cast<LPBYTE>(pData);
	TCHAR strTemp[3] = {0};

	for (DWORD j = 0; j < size; ++j) {
		BYTE ch = *ptr;
		strTemp[1] = (ch & 0xf) > 9 ? '7'+ (ch & 0xf) : '0' + (ch & 0xf);
		ch = ch >> 4;

		strTemp[0] = (ch & 0xf) > 9 ? '7'+ (ch & 0xf) : '0' + (ch & 0xf);
		ptr++;

		bstr += strTemp;
	}

	*pStream = bstr.Detach ();
}

//
//	Function:
//		Extract (LPCWSTR *pStream, void *pData, DWORD size)
//
//	Parameters:
//		pStream		Pointer to LPWSTR
//		pData		Pointer to buffer
//		size		Size of buffer in bytes
//
//	Return Values:
//
//	Comments;
//		It takes LPWSTR pointer and extracts size two-byte hex strings, converting each to a byte
//		and then placing them in the buffer. The position in the input stream in updated to
//		reflect how many characters have been used.
//
//	NOTE: For more information see "Professional ATL COM Programming, Wrox Press Ltd" by
//		  Dr. Richard Grimes.
//

void
CShapeCtl::Extract (LPCWSTR *pStream, void *pData, DWORD size)
{
	LPCWSTR str = *pStream;
	LPBYTE ptr = static_cast<LPBYTE>(pData);

	for (DWORD j = 0; j < size; ++j) {
		BYTE ch = (str[0] >= L'A') ? str[0] - L'7' : str[0] - L'0';
		ch *= 16;
		ch += (str[1] >= L'A') ? str[1] - L'7' : str[1] - L'0';
		*ptr = ch;
		ptr++;
		str += 2;
	}

	*pStream = str;
}
